@@ -8,11 +8,9 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package MooseX::AttributeShortcuts;
-BEGIN {
-  $MooseX::AttributeShortcuts::AUTHORITY = 'cpan:RSRCHBOY';
-}
-# git description: 0.022-7-g4a1fc05
-$MooseX::AttributeShortcuts::VERSION = '0.023';
+our $AUTHORITY = 'cpan:RSRCHBOY';
+# git description: 0.026-4-g9655ffa
+$MooseX::AttributeShortcuts::VERSION = '0.027';
 
 # ABSTRACT: Shorthand for common attribute options
 
@@ -28,17 +26,22 @@ use Moose::Util::TypeConstraints;
 
 {
     package MooseX::AttributeShortcuts::Trait::Attribute;
-BEGIN {
-  $MooseX::AttributeShortcuts::Trait::Attribute::AUTHORITY = 'cpan:RSRCHBOY';
-}
-# git description: 0.022-7-g4a1fc05
-$MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.023';
+our $AUTHORITY = 'cpan:RSRCHBOY';
+# git description: 0.026-4-g9655ffa
+$MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.027';
     use namespace::autoclean;
     use MooseX::Role::Parameterized;
     use Moose::Util::TypeConstraints  ':all';
     use MooseX::Types::Moose          ':all';
     use MooseX::Types::Common::String ':all';
 
+    use List::AllUtils 'any';
+
+    use Package::DeprecationManager -deprecations => {
+        'undocumented-isa-constraints' => '0.23',
+        'hashref-given-to-coerce'      => '0.24',
+    };
+
     sub _acquire_isa_tc { goto \&Moose::Util::TypeConstraints::find_or_create_isa_type_constraint }
 
     parameter writer_prefix  => (isa => NonEmptySimpleStr, default => '_set_');
@@ -95,12 +98,14 @@ $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.023';
 
             if ($options->{is}) {
 
+                # handle: is => 'rwp'
                 if ($options->{is} eq 'rwp') {
 
                     $options->{is}     = 'ro';
                     $options->{writer} = "$wprefix$name";
                 }
 
+                # handle: is => 'lazy'
                 if ($options->{is} eq 'lazy') {
 
                     $options->{is}       = 'ro';
@@ -110,22 +115,22 @@ $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.023';
                 }
             }
 
-            # TODO isa_class - anon class_type generation
-            # TODO isa_role  - anon role_type generation
-            # TODO isa_enum  - anon enum generation
-            # TODO coerce_via - anon coercion (type -> anon subtype+coercion
+            if (any { exists $options->{$_} } (qw{ isa_class isa_role isa_enum })) {
 
-            # XXX we also ignore conflicts here -- last in wins
-            #confess q{conflict 'isa' and 'isa_class' or 'isa_role'}
-                #if $_has->('isa')
+                # (more than) fair warning...
+                deprecated(
+                    feature => 'undocumented-isa-constraints',
+                    message => 'Naughty! isa_class, isa_role, and isa_enum will be removed on or after 01 July 2015!',
+                );
 
-            # XXX undocumented -- not sure this is a great idea
-            $options->{isa} = class_type(delete $options->{isa_class})
-                if $_has->('isa_class');
-            $options->{isa} = role_type(delete $options->{isa_role})
-                if $_has->('isa_role');
-            $options->{isa} = enum(delete $options->{isa_enum})
-                if $_has->('isa_enum');
+                # XXX undocumented -- not sure this is a great idea
+                $options->{isa} = class_type(delete $options->{isa_class})
+                    if $_has->('isa_class');
+                $options->{isa} = role_type(delete $options->{isa_role})
+                    if $_has->('isa_role');
+                $options->{isa} = enum(delete $options->{isa_enum})
+                    if $_has->('isa_enum');
+            }
 
             # aka: isa => class_type(...)
             if ($_has->('isa_instance_of')) {
@@ -162,9 +167,15 @@ $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.023';
             }
 
             # "fix" the case of the hashref....  *sigh*
-            # FIXME TODO check for potential conflicts and warn!
-            $options->{coerce} = [ %{ $options->{coerce} } ]
-                if $_ref->('coerce') eq 'HASH';
+            if ($_ref->('coerce') eq 'HASH') {
+
+                deprecated(
+                    feature => 'hashref-given-to-coerce',
+                    message => 'Passing a hashref to coerce is unsafe, and will be removed on or after 01 Jan 2015',
+                );
+
+                $options->{coerce} = [ %{ $options->{coerce} } ];
+            }
 
             if ($_ref->('coerce') eq 'ARRAY') {
 
@@ -283,6 +294,36 @@ $MooseX::AttributeShortcuts::Trait::Attribute::VERSION = '0.023';
             $class->add_method($self->builder => $self->anon_builder);
             return;
         };
+
+        method mi               => sub { shift->associated_class->get_meta_instance                     };
+        method weaken_value     => sub { $_[0]->mi->weaken_slot_value($_[1] => $_) for $_[0]->slots     };
+        method strengthen_value => sub { $_[0]->mi->strengthen_slot_value($_[1] => $_) for $_[0]->slots };
+
+        # NOTE: remove_delegation() will also automagically remove any custom
+        # accessors we create here
+
+        around _make_delegation_method => sub {
+            my ($orig, $self) = (shift, shift);
+            my ($name, $coderef) = @_;
+
+            ### _make_delegation_method() called with a: ref $coderef
+            return $self->$orig(@_)
+                unless 'CODE' eq ref $coderef;
+
+            # this coderef will be installed as a method on the associated class itself.
+            my $custom_coderef = sub {
+                # aka $self from the class instance's perspective
+                my $associated_class_instance = shift @_;
+
+                # in $coderef, $_ will be the attribute metaclass
+                local $_ = $self;
+                return $associated_class_instance->$coderef(@_);
+            };
+
+            return $self->_process_accessors(custom => { $name => $custom_coderef });
+        };
+
+        return;
     };
 }
 
@@ -354,8 +395,10 @@ __END__
 
 =encoding UTF-8
 
-=for :stopwords Chris Weyl David Steinbrunner <dsteinbrunner@pobox.com> GitHub attribute's
-isa one's rwp SUBTYPING foo
+=for :stopwords Chris Weyl David Graham Knop Steinbrunner GitHub attribute's isa one's rwp
+SUBTYPING foo
+
+=for :stopwords Wishlist flattr flattr'ed gittip gittip'ed
 
 =head1 NAME
 
@@ -363,7 +406,7 @@ MooseX::AttributeShortcuts - Shorthand for common attribute options
 
 =head1 VERSION
 
-This document describes version 0.023 of MooseX::AttributeShortcuts - released April 04, 2014 as part of MooseX-AttributeShortcuts.
+This document describes version 0.027 of MooseX::AttributeShortcuts - released March 10, 2015 as part of MooseX-AttributeShortcuts.
 
 =head1 SYNOPSIS
 
@@ -631,8 +674,52 @@ coderefs that will coerce a given type to our type.
         ],
     );
 
+=head2 handles => { foo => sub { ... }, ... }
+
+Creating a delegation with a coderef will now create a new, "custom accessor"
+for the attribute.  These coderefs will be installed and called as methods on
+the associated class (just as readers, writers, and other accessors are), and
+will have the attribute metaclass available in $_.  Anything the accessor
+is called with it will have access to in @_, just as you'd expect of a method.
+
+e.g., the following example creates an attribute named 'bar' with a standard
+reader accessor named 'bar' and two custom accessors named 'foo' and
+'foo_too'.
+
+    has bar => (
+
+        is      => 'ro',
+        isa     => 'Int',
+        handles => {
+
+            foo => sub {
+                my $self = shift @_;
+
+                return $_->get_value($self) + 1;
+            },
+
+            foo_too => sub {
+                my $self = shift @_;
+
+                return $self->bar + 1;
+            },
+        },
+    );
+
+...and later,
+
+Note that in this example both foo() and foo_too() do effectively the same
+thing: return the attribute's current value plus 1.  However, foo() accesses
+the attribute value directly through the metaclass, the pros and cons of
+which this author leaves as an exercise for the reader to determine.
+
+You may choose to use the installed accessors to get at the attribute's value,
+or use the direct metaclass access, your choice.
+
 =head1 ANONYMOUS SUBTYPING AND COERCION
 
+    "Abusus non tollit usum."
+
 Note that we create new, anonymous subtypes whenever the constraint or
 coercion options are specified in such a way that the Shortcuts trait (this
 one) is invoked.  It's fully supported to use both constraint and coerce
@@ -660,8 +747,8 @@ L<MooseX::Types|MooseX::Types>
 
 =head1 SOURCE
 
-The development version is on github at L<http://github.com/RsrchBoy/moosex-attributeshortcuts>
-and may be cloned from L<git://github.com/RsrchBoy/moosex-attributeshortcuts.git>
+The development version is on github at L<http://https://github.com/RsrchBoy/moosex-attributeshortcuts>
+and may be cloned from L<git://https://github.com/RsrchBoy/moosex-attributeshortcuts.git>
 
 =head1 BUGS
 
@@ -676,10 +763,41 @@ feature.
 
 Chris Weyl <cweyl@alumni.drew.edu>
 
-=head1 CONTRIBUTOR
+=head2 I'm a material boy in a material world
+
+=begin html
+
+<a href="https://www.gittip.com/RsrchBoy/"><img src="https://raw.githubusercontent.com/gittip/www.gittip.com/master/www/assets/%25version/logo.png" /></a>
+<a href="http://bit.ly/rsrchboys-wishlist"><img src="http://wps.io/wp-content/uploads/2014/05/amazon_wishlist.resized.png" /></a>
+<a href="https://flattr.com/submit/auto?user_id=RsrchBoy&url=https%3A%2F%2Fgithub.com%2FRsrchBoy%2Fmoosex-attributeshortcuts&title=RsrchBoy's%20CPAN%20MooseX-AttributeShortcuts&tags=%22RsrchBoy's%20MooseX-AttributeShortcuts%20in%20the%20CPAN%22"><img src="http://api.flattr.com/button/flattr-badge-large.png" /></a>
+
+=end html
+
+Please note B<I do not expect to be gittip'ed or flattr'ed for this work>,
+rather B<it is simply a very pleasant surprise>. I largely create and release
+works like this because I need them or I find it enjoyable; however, don't let
+that stop you if you feel like it ;)
+
+L<Flattr this|https://flattr.com/submit/auto?user_id=RsrchBoy&url=https%3A%2F%2Fgithub.com%2FRsrchBoy%2Fmoosex-attributeshortcuts&title=RsrchBoy's%20CPAN%20MooseX-AttributeShortcuts&tags=%22RsrchBoy's%20MooseX-AttributeShortcuts%20in%20the%20CPAN%22>,
+L<gittip me|https://www.gittip.com/RsrchBoy/>, or indulge my
+L<Amazon Wishlist|http://bit.ly/rsrchboys-wishlist>...  If you so desire.
+
+=head1 CONTRIBUTORS
+
+=for stopwords David Steinbrunner Graham Knop
+
+=over 4
+
+=item *
 
 David Steinbrunner <dsteinbrunner@pobox.com>
 
+=item *
+
+Graham Knop <haarg@haarg.org>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is Copyright (c) 2011 by Chris Weyl.
